==> core-0.0.6-sources.jar: cn/bestwu/simpleframework/support/excel/ExcelExport.java <==
package cn.bestwu.simpleframework.support.excel;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.mail.internet.MimeUtility;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Comment;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.StringUtils;

/**
 * 导出Excel文件（导出“XLSX”格式，支持大数据量导出   @see org.apache.poi.ss.SpreadsheetVersion）
 */
public class ExcelExport {

  private static Logger log = LoggerFactory.getLogger(ExcelExport.class);
  /**
   * 注解列表（Object[]{ ExcelField, Field/Method }）
   */
  List<Object[]> annotationList = new ArrayList<>();
  /**
   * 工作薄对象
   */
  private SXSSFWorkbook wb;
  /**
   * 工作表对象
   */
  private SXSSFSheet sheet;
  /**
   * 样式列表
   */
  private Map<String, CellStyle> styles;
  /**
   * 当前行号
   */
  private int rownum;

  /**
   * 构造函数
   *
   * @param cls 实体对象，通过annotation.ExportField获取标题
   */
  public ExcelExport(Class<?> cls) {
    this(null, cls, 1);
  }

  /**
   * 构造函数
   *
   * @param cls 实体对象，通过annotation.ExportField获取标题
   * @param type 导出类型（1:导出数据；2：导出模板）
   */
  public ExcelExport(Class<?> cls, int type) {
    this(null, cls, type);
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param cls 实体对象，通过annotation.ExportField获取标题
   */
  public ExcelExport(String title, Class<?> cls) {
    this(title, cls, 1);
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param cls 实体对象，通过annotation.ExportField获取标题
   * @param type 导出类型（1:导出数据；2：导出模板）
   * @param groups 导入分组
   */
  public ExcelExport(String title, Class<?> cls, int type, int... groups) {
    // Get annotation field
    Field[] fs = cls.getDeclaredFields();
    for (Field f : fs) {
      ExcelField ef = f.getAnnotation(ExcelField.class);
      if (ef != null && (ef.type() == 0 || ef.type() == type)) {
        if (groups != null && groups.length > 0) {
          boolean inGroup = false;
          for (int g : groups) {
            if (inGroup) {
              break;
            }
            for (int efg : ef.groups()) {
              if (g == efg) {
                inGroup = true;
                annotationList.add(new Object[]{ef, f});
                break;
              }
            }
          }
        } else {
          annotationList.add(new Object[]{ef, f});
        }
      }
    }
    // Get annotation method
    Method[] ms = cls.getDeclaredMethods();
    for (Method m : ms) {
      ExcelField ef = m.getAnnotation(ExcelField.class);
      if (ef != null && (ef.type() == 0 || ef.type() == type)) {
        if (groups != null && groups.length > 0) {
          boolean inGroup = false;
          for (int g : groups) {
            if (inGroup) {
              break;
            }
            for (int efg : ef.groups()) {
              if (g == efg) {
                inGroup = true;
                annotationList.add(new Object[]{ef, m});
                break;
              }
            }
          }
        } else {
          annotationList.add(new Object[]{ef, m});
        }
      }
    }
    // Field sorting
    annotationList.sort(Comparator.comparingInt(o -> ((ExcelField) o[0]).sort()));
    // Initialize
    List<String> headerList = new ArrayList<>();
    for (Object[] os : annotationList) {
      String t = ((ExcelField) os[0]).title();
      // 如果是导出，则去掉注释
      if (type == 1) {
        String[] ss = StringUtils.split(t, "**");
        if (ss != null && ss.length == 2) {
          t = ss[0];
        }
      }
      headerList.add(t);
    }
    initialize(title, headerList);
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param headers 表头数组
   */
  public ExcelExport(String title, String[] headers) {
    initialize(title, new ArrayList<>(Arrays.asList(headers)));
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param headerList 表头列表
   */
  public ExcelExport(String title, List<String> headerList) {
    initialize(title, headerList);
  }

  /**
   * 初始化函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param headerList 表头列表
   */
  private void initialize(String title, List<String> headerList) {
    this.wb = new SXSSFWorkbook(500);
    this.sheet = wb.createSheet("Export");
    this.styles = createStyles(wb);
    // Create title
    if (StringUtils.hasText(title)) {
      Row titleRow = sheet.createRow(rownum++);
      titleRow.setHeightInPoints(30);
      Cell titleCell = titleRow.createCell(0);
      titleCell.setCellStyle(styles.get("title"));
      titleCell.setCellValue(title);
      sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(),
          titleRow.getRowNum(), titleRow.getRowNum(), headerList.size() - 1));
    }
    // Create header
    if (headerList == null) {
      throw new RuntimeException("headerList not null!");
    }
    Row headerRow = sheet.createRow(rownum++);
    headerRow.setHeightInPoints(16);
    for (int i = 0; i < headerList.size(); i++) {
      Cell cell = headerRow.createCell(i);
      cell.setCellStyle(styles.get("header"));
      String[] ss = StringUtils.split(headerList.get(i), "**");
      if (ss != null && ss.length == 2) {
        cell.setCellValue(ss[0]);
        Comment comment = this.sheet.createDrawingPatriarch().createCellComment(
            new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
        comment.setString(new XSSFRichTextString(ss[1]));
        cell.setCellComment(comment);
      } else {
        cell.setCellValue(headerList.get(i));
      }
      sheet.trackAllColumnsForAutoSizing();
      sheet.autoSizeColumn(i);
    }
    for (int i = 0; i < headerList.size(); i++) {
      int colWidth = sheet.getColumnWidth(i) * 2;
      sheet.setColumnWidth(i, colWidth < 3000 ? 3000 : colWidth);
    }
    log.debug("Initialize success.");
  }

  /**
   * 创建表格样式
   *
   * @param wb 工作薄对象
   * @return 样式列表
   */
  private Map<String, CellStyle> createStyles(Workbook wb) {
    Map<String, CellStyle> styles = new HashMap<>();

    CellStyle style = wb.createCellStyle();
    style.setAlignment(HorizontalAlignment.CENTER);
    style.setVerticalAlignment(VerticalAlignment.CENTER);
    Font titleFont = wb.createFont();
    titleFont.setFontName("Arial");
    titleFont.setFontHeightInPoints((short) 16);
    titleFont.setBold(true);
    style.setFont(titleFont);
    styles.put("title", style);

    style = wb.createCellStyle();
    style.setVerticalAlignment(VerticalAlignment.CENTER);
    style.setBorderRight(BorderStyle.THIN);
    style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setBorderLeft(BorderStyle.THIN);
    style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setBorderTop(BorderStyle.THIN);
    style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setBorderBottom(BorderStyle.THIN);
    style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    Font dataFont = wb.createFont();
    dataFont.setFontName("Arial");
    dataFont.setFontHeightInPoints((short) 10);
    style.setFont(dataFont);
    styles.put("data", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
    style.setAlignment(HorizontalAlignment.LEFT);
    styles.put("data1", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
    style.setAlignment(HorizontalAlignment.CENTER);
    styles.put("data2", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
    style.setAlignment(HorizontalAlignment.RIGHT);
    styles.put("data3", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
//		style.setWrapText(true);
    style.setAlignment(HorizontalAlignment.CENTER);
    style.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
    Font headerFont = wb.createFont();
    headerFont.setFontName("Arial");
    headerFont.setFontHeightInPoints((short) 10);
    headerFont.setBold(true);
    headerFont.setColor(IndexedColors.WHITE.getIndex());
    style.setFont(headerFont);
    styles.put("header", style);

    return styles;
  }

  /**
   * 添加一行
   *
   * @return 行对象
   */
  public Row addRow() {
    return sheet.createRow(rownum++);
  }


  /**
   * 添加一个单元格
   *
   * @param row 添加的行
   * @param column 添加列号
   * @param val 添加值
   * @return 单元格对象
   */
  public Cell addCell(Row row, int column, Object val) {
    return this.addCell(row, column, val, 0, Class.class);
  }

  /**
   * 添加一个单元格
   *
   * @param row 添加的行
   * @param column 添加列号
   * @param val 添加值
   * @param align 对齐方式（1：靠左；2：居中；3：靠右）
   * @param fieldType fieldType
   * @return 单元格对象
   */
  public Cell addCell(Row row, int column, Object val, int align, Class<?> fieldType) {
    Cell cell = row.createCell(column);
    String cellFormatString = "@";
    try {
      if (val == null) {
        cell.setCellValue("");
      } else if (fieldType != Class.class) {
        cell.setCellValue((String) fieldType.getMethod("setValue", Object.class).invoke(null, val));
      } else {
        if (val instanceof String) {
          cell.setCellValue((String) val);
        } else if (val instanceof Integer) {
          cell.setCellValue((Integer) val);
          cellFormatString = "0";
        } else if (val instanceof Long) {
          cell.setCellValue((Long) val);
          cellFormatString = "0";
        } else if (val instanceof Double) {
          cell.setCellValue((Double) val);
          cellFormatString = "0.00";
        } else if (val instanceof Float) {
          cell.setCellValue((Float) val);
          cellFormatString = "0.00";
        } else if (val instanceof Date) {
          cell.setCellValue((Date) val);
          cellFormatString = "yyyy-MM-dd HH:mm";
        } else {
          cell.setCellValue((String) Class
              .forName(this.getClass().getName().replaceAll(this.getClass().getSimpleName(),
                  "fieldtype." + val.getClass().getSimpleName() + "Type"))
              .getMethod("setValue", Object.class).invoke(null, val));
        }
      }
      if (val != null) {
        CellStyle style = styles.get("data_column_" + column);
        if (style == null) {
          style = wb.createCellStyle();
          style.cloneStyleFrom(styles.get("data" + (align >= 1 && align <= 3 ? align : "")));
          style.setDataFormat(wb.createDataFormat().getFormat(cellFormatString));
          styles.put("data_column_" + column, style);
        }
        cell.setCellStyle(style);
      }
    } catch (Exception ex) {
      log.info("Set cell value [" + row.getRowNum() + "," + column + "] error: " + ex.toString());
      cell.setCellValue(val.toString());
    }
    return cell;
  }

  /**
   * 调用Getter方法.
   * 支持多级，如：对象名.对象名.方法
   */
  private Object invokeGetter(Object obj, String propertyName) throws NoSuchMethodException {
    Object object = obj;
    for (String name : StringUtils.split(propertyName, ".")) {
      String getterMethodName = "get" + StringUtils.capitalize(name);
      object = ReflectionUtils.invokeMethod(object.getClass().getMethod(getterMethodName), object);
    }
    return object;
  }

  /**
   * 添加数据（通过annotation.ExportField添加数据）
   *
   * @param <E> E
   * @param list list
   * @return list 数据列表
   */
  public <E> ExcelExport setDataList(List<E> list) {
    for (E e : list) {
      int colunm = 0;
      Row row = this.addRow();
      StringBuilder sb = new StringBuilder();
      for (Object[] os : annotationList) {
        ExcelField ef = (ExcelField) os[0];
        Object val = null;
        // Get entity value
        try {
          if (StringUtils.hasText(ef.value())) {
            val = invokeGetter(e, ef.value());
          } else {
            if (os[1] instanceof Field) {
              val = ReflectionUtils.invokeMethod(
                  BeanUtils.getPropertyDescriptor(e.getClass(), ((Field) os[1]).getName())
                      .getReadMethod(), e);
            } else if (os[1] instanceof Method) {
              val = ReflectionUtils.invokeMethod((Method) os[1], e);
            }
          }
        } catch (Exception ex) {
          // Failure to ignore
          log.info(ex.toString());
          val = "";
        }
        this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
        sb.append(val).append(", ");
      }
      log.debug("Write success: [" + row.getRowNum() + "] " + sb.toString());
    }
    return this;
  }

  /**
   * 输出数据流
   *
   * @param os 输出数据流
   * @return this
   * @throws IOException IOException
   */
  public ExcelExport write(OutputStream os) throws IOException {
    wb.write(os);
    return this;
  }

  /**
   * 输出到客户端
   *
   * @param request request
   * @param response response
   * @param fileName 输出文件名
   * @return this
   * @throws IOException IOException
   */
  public ExcelExport write(HttpServletRequest request, HttpServletResponse response,
      String fileName) throws IOException {
    response.reset();
    String agent = request.getHeader("USER-AGENT");

    if (null != agent && (agent.contains("Trident") || agent.contains("Edge"))) {
      fileName = URLEncoder.encode(fileName, "UTF-8");
    } else {
      fileName = MimeUtility.encodeText(fileName, "UTF8", "B");
    }
    response.setHeader("Content-Disposition", "attachment; filename=" + fileName + ".xlsx");
    response.setContentType("application/vnd.ms-excel; charset=utf-8");
    response.setHeader("Pragma", "No-cache");
    response.setHeader("Cache-Control", "no-cache");
    response.setDateHeader("Expires", 0);
    write(response.getOutputStream());
    return this;
  }

  /**
   * 输出到文件
   *
   * @param fileName 输出文件名
   * @return this
   * @throws IOException IOException
   */
  public ExcelExport writeFile(String fileName) throws IOException {
    FileOutputStream os = new FileOutputStream(fileName);
    this.write(os);
    return this;
  }

  /**
   * 清理临时文件
   *
   * @return this
   */
  public ExcelExport dispose() {
    wb.dispose();
    return this;
  }

}

==> core-0.1.0-sources.jar: cn/bestwu/simpleframework/support/excel/ExcelExport.java <==
package cn.bestwu.simpleframework.support.excel;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.mail.internet.MimeUtility;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Comment;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.StringUtils;

/**
 * 导出Excel文件（导出“XLSX”格式，支持大数据量导出   @see org.apache.poi.ss.SpreadsheetVersion）
 */
public class ExcelExport {

  private static Logger log = LoggerFactory.getLogger(ExcelExport.class);
  /**
   * 注解列表（Object[]{ ExcelField, Field/Method }）
   */
  List<Object[]> annotationList = new ArrayList<>();
  /**
   * 工作薄对象
   */
  private SXSSFWorkbook wb;
  /**
   * 工作表对象
   */
  private SXSSFSheet sheet;
  /**
   * 样式列表
   */
  private Map<String, CellStyle> styles;
  /**
   * 当前行号
   */
  private int rownum;

  /**
   * 构造函数
   *
   * @param cls 实体对象，通过annotation.ExportField获取标题
   */
  public ExcelExport(Class<?> cls) {
    this(null, cls, 1);
  }

  /**
   * 构造函数
   *
   * @param cls 实体对象，通过annotation.ExportField获取标题
   * @param type 导出类型（1:导出数据；2：导出模板）
   */
  public ExcelExport(Class<?> cls, int type) {
    this(null, cls, type);
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param cls 实体对象，通过annotation.ExportField获取标题
   */
  public ExcelExport(String title, Class<?> cls) {
    this(title, cls, 1);
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param cls 实体对象，通过annotation.ExportField获取标题
   * @param type 导出类型（1:导出数据；2：导出模板）
   * @param groups 导入分组
   */
  public ExcelExport(String title, Class<?> cls, int type, int... groups) {
    // Get annotation field
    Field[] fs = cls.getDeclaredFields();
    for (Field f : fs) {
      ExcelField ef = f.getAnnotation(ExcelField.class);
      if (ef != null && (ef.type() == 0 || ef.type() == type)) {
        if (groups != null && groups.length > 0) {
          boolean inGroup = false;
          for (int g : groups) {
            if (inGroup) {
              break;
            }
            for (int efg : ef.groups()) {
              if (g == efg) {
                inGroup = true;
                annotationList.add(new Object[]{ef, f});
                break;
              }
            }
          }
        } else {
          annotationList.add(new Object[]{ef, f});
        }
      }
    }
    // Get annotation method
    Method[] ms = cls.getDeclaredMethods();
    for (Method m : ms) {
      ExcelField ef = m.getAnnotation(ExcelField.class);
      if (ef != null && (ef.type() == 0 || ef.type() == type)) {
        if (groups != null && groups.length > 0) {
          boolean inGroup = false;
          for (int g : groups) {
            if (inGroup) {
              break;
            }
            for (int efg : ef.groups()) {
              if (g == efg) {
                inGroup = true;
                annotationList.add(new Object[]{ef, m});
                break;
              }
            }
          }
        } else {
          annotationList.add(new Object[]{ef, m});
        }
      }
    }
    // Field sorting
    annotationList.sort(Comparator.comparingInt(o -> ((ExcelField) o[0]).sort()));
    // Initialize
    List<String> headerList = new ArrayList<>();
    for (Object[] os : annotationList) {
      String t = ((ExcelField) os[0]).title();
      // 如果是导出，则去掉注释
      if (type == 1) {
        String[] ss = StringUtils.split(t, "**");
        if (ss != null && ss.length == 2) {
          t = ss[0];
        }
      }
      headerList.add(t);
    }
    initialize(title, headerList);
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param headers 表头数组
   */
  public ExcelExport(String title, String[] headers) {
    initialize(title, new ArrayList<>(Arrays.asList(headers)));
  }

  /**
   * 构造函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param headerList 表头列表
   */
  public ExcelExport(String title, List<String> headerList) {
    initialize(title, headerList);
  }

  /**
   * 初始化函数
   *
   * @param title 表格标题，传“空值”，表示无标题
   * @param headerList 表头列表
   */
  private void initialize(String title, List<String> headerList) {
    this.wb = new SXSSFWorkbook(500);
    this.sheet = wb.createSheet("Export");
    this.styles = createStyles(wb);
    // Create title
    if (StringUtils.hasText(title)) {
      Row titleRow = sheet.createRow(rownum++);
      titleRow.setHeightInPoints(30);
      Cell titleCell = titleRow.createCell(0);
      titleCell.setCellStyle(styles.get("title"));
      titleCell.setCellValue(title);
      sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(),
          titleRow.getRowNum(), titleRow.getRowNum(), headerList.size() - 1));
    }
    // Create header
    if (headerList == null) {
      throw new RuntimeException("headerList not null!");
    }
    Row headerRow = sheet.createRow(rownum++);
    headerRow.setHeightInPoints(16);
    for (int i = 0; i < headerList.size(); i++) {
      Cell cell = headerRow.createCell(i);
      cell.setCellStyle(styles.get("header"));
      String[] ss = StringUtils.split(headerList.get(i), "**");
      if (ss != null && ss.length == 2) {
        cell.setCellValue(ss[0]);
        Comment comment = this.sheet.createDrawingPatriarch().createCellComment(
            new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
        comment.setString(new XSSFRichTextString(ss[1]));
        cell.setCellComment(comment);
      } else {
        cell.setCellValue(headerList.get(i));
      }
      sheet.trackAllColumnsForAutoSizing();
      sheet.autoSizeColumn(i);
    }
    for (int i = 0; i < headerList.size(); i++) {
      int colWidth = sheet.getColumnWidth(i) * 2;
      sheet.setColumnWidth(i, colWidth < 3000 ? 3000 : colWidth);
    }
    log.debug("Initialize success.");
  }

  /**
   * 创建表格样式
   *
   * @param wb 工作薄对象
   * @return 样式列表
   */
  private Map<String, CellStyle> createStyles(Workbook wb) {
    Map<String, CellStyle> styles = new HashMap<>();

    CellStyle style = wb.createCellStyle();
    style.setAlignment(HorizontalAlignment.CENTER);
    style.setVerticalAlignment(VerticalAlignment.CENTER);
    Font titleFont = wb.createFont();
    titleFont.setFontName("Arial");
    titleFont.setFontHeightInPoints((short) 16);
    titleFont.setBold(true);
    style.setFont(titleFont);
    styles.put("title", style);

    style = wb.createCellStyle();
    style.setVerticalAlignment(VerticalAlignment.CENTER);
    style.setBorderRight(BorderStyle.THIN);
    style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setBorderLeft(BorderStyle.THIN);
    style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setBorderTop(BorderStyle.THIN);
    style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setBorderBottom(BorderStyle.THIN);
    style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
    Font dataFont = wb.createFont();
    dataFont.setFontName("Arial");
    dataFont.setFontHeightInPoints((short) 10);
    style.setFont(dataFont);
    styles.put("data", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
    style.setAlignment(HorizontalAlignment.LEFT);
    styles.put("data1", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
    style.setAlignment(HorizontalAlignment.CENTER);
    styles.put("data2", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
    style.setAlignment(HorizontalAlignment.RIGHT);
    styles.put("data3", style);

    style = wb.createCellStyle();
    style.cloneStyleFrom(styles.get("data"));
//		style.setWrapText(true);
    style.setAlignment(HorizontalAlignment.CENTER);
    style.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
    style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
    Font headerFont = wb.createFont();
    headerFont.setFontName("Arial");
    headerFont.setFontHeightInPoints((short) 10);
    headerFont.setBold(true);
    headerFont.setColor(IndexedColors.WHITE.getIndex());
    style.setFont(headerFont);
    styles.put("header", style);

    return styles;
  }

  /**
   * 添加一行
   *
   * @return 行对象
   */
  public Row addRow() {
    return sheet.createRow(rownum++);
  }


  /**
   * 添加一个单元格
   *
   * @param row 添加的行
   * @param column 添加列号
   * @param val 添加值
   * @return 单元格对象
   */
  public Cell addCell(Row row, int column, Object val) {
    return this.addCell(row, column, val, 0, Class.class);
  }

  /**
   * 添加一个单元格
   *
   * @param row 添加的行
   * @param column 添加列号
   * @param val 添加值
   * @param align 对齐方式（1：靠左；2：居中；3：靠右）
   * @param fieldType fieldType
   * @return 单元格对象
   */
  public Cell addCell(Row row, int column, Object val, int align, Class<?> fieldType) {
    Cell cell = row.createCell(column);
    String cellFormatString = "@";
    try {
      if (val == null) {
        cell.setCellValue("");
      } else if (fieldType != Class.class) {
        cell.setCellValue((String) fieldType.getMethod("setValue", Object.class).invoke(null, val));
      } else {
        if (val instanceof String) {
          cell.setCellValue((String) val);
        } else if (val instanceof Integer) {
          cell.setCellValue((Integer) val);
          cellFormatString = "0";
        } else if (val instanceof Long) {
          cell.setCellValue((Long) val);
          cellFormatString = "0";
        } else if (val instanceof Double) {
          cell.setCellValue((Double) val);
          cellFormatString = "0.00";
        } else if (val instanceof Float) {
          cell.setCellValue((Float) val);
          cellFormatString = "0.00";
        } else if (val instanceof Date) {
          cell.setCellValue((Date) val);
          cellFormatString = "yyyy-MM-dd HH:mm";
        } else {
          cell.setCellValue((String) Class
              .forName(this.getClass().getName().replaceAll(this.getClass().getSimpleName(),
                  "fieldtype." + val.getClass().getSimpleName() + "Type"))
              .getMethod("setValue", Object.class).invoke(null, val));
        }
      }
      if (val != null) {
        CellStyle style = styles.get("data_column_" + column);
        if (style == null) {
          style = wb.createCellStyle();
          style.cloneStyleFrom(styles.get("data" + (align >= 1 && align <= 3 ? align : "")));
          style.setDataFormat(wb.createDataFormat().getFormat(cellFormatString));
          styles.put("data_column_" + column, style);
        }
        cell.setCellStyle(style);
      }
    } catch (Exception ex) {
      log.info("Set cell value [" + row.getRowNum() + "," + column + "] error: " + ex.toString());
      cell.setCellValue(val.toString());
    }
    return cell;
  }

  /**
   * 调用Getter方法.
   * 支持多级，如：对象名.对象名.方法
   */
  private Object invokeGetter(Object obj, String propertyName) throws NoSuchMethodException {
    Object object = obj;
    for (String name : StringUtils.split(propertyName, ".")) {
      String getterMethodName = "get" + StringUtils.capitalize(name);
      object = ReflectionUtils.invokeMethod(object.getClass().getMethod(getterMethodName), object);
    }
    return object;
  }

  /**
   * 添加数据（通过annotation.ExportField添加数据）
   *
   * @param <E> E
   * @param list list
   * @return list 数据列表
   */
  public <E> ExcelExport setDataList(List<E> list) {
    for (E e : list) {
      int colunm = 0;
      Row row = this.addRow();
      StringBuilder sb = new StringBuilder();
      for (Object[] os : annotationList) {
        ExcelField ef = (ExcelField) os[0];
        Object val = null;
        // Get entity value
        try {
          if (StringUtils.hasText(ef.value())) {
            val = invokeGetter(e, ef.value());
          } else {
            if (os[1] instanceof Field) {
              val = ReflectionUtils.invokeMethod(
                  BeanUtils.getPropertyDescriptor(e.getClass(), ((Field) os[1]).getName())
                      .getReadMethod(), e);
            } else if (os[1] instanceof Method) {
              val = ReflectionUtils.invokeMethod((Method) os[1], e);
            }
          }
        } catch (Exception ex) {
          // Failure to ignore
          log.info(ex.toString());
          val = "";
        }
        this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
        sb.append(val).append(", ");
      }
      log.debug("Write success: [" + row.getRowNum() + "] " + sb.toString());
    }
    return this;
  }

  /**
   * 输出数据流
   *
   * @param os 输出数据流
   * @return this
   * @throws IOException IOException
   */
  public ExcelExport write(OutputStream os) throws IOException {
    wb.write(os);
    return this;
  }

  /**
   * 输出到客户端
   *
   * @param request request
   * @param response response
   * @param fileName 输出文件名
   * @return this
   * @throws IOException IOException
   */
  public ExcelExport write(HttpServletRequest request, HttpServletResponse response,
      String fileName) throws IOException {
    response.reset();
    String agent = request.getHeader("USER-AGENT");

    String newFileName;
    if (null != agent && (agent.contains("Trident") || agent.contains("Edge"))) {
      newFileName = URLEncoder.encode(fileName, "UTF-8");
    } else {
      newFileName = MimeUtility.encodeText(fileName, "UTF8", "B");
    }
    response.setHeader("Content-Disposition", "attachment;filename=" + newFileName + ".xlsx;filename*=UTF-8''" + URLEncoder.encode(fileName, "UTF-8") + ".xlsx");
    response.setContentType("application/vnd.ms-excel; charset=utf-8");
    response.setHeader("Pragma", "No-cache");
    response.setHeader("Cache-Control", "no-cache");
    response.setDateHeader("Expires", 0);
    write(response.getOutputStream());
    return this;
  }

  /**
   * 输出到文件
   *
   * @param fileName 输出文件名
   * @return this
   * @throws IOException IOException
   */
  public ExcelExport writeFile(String fileName) throws IOException {
    FileOutputStream os = new FileOutputStream(fileName);
    this.write(os);
    return this;
  }

  /**
   * 清理临时文件
   *
   * @return this
   */
  public ExcelExport dispose() {
    wb.dispose();
    return this;
  }

}
